//Code Example: mainClient.cc
// Author: Alberto D'Angeli


#include <fstream>

#include <unistd.h>
#include <stdlib.h>

// Include files generated from our IDL
#include <it/mokabyte/eai/my/ejb/Calculator.h>
#include <it/mokabyte/eai/my/ejb/CalculatorHome.h>

using namespace std;


void
run(CORBA::ORB_ptr orb, const char* home_url)
{
	
  cout << "Looking for: " << home_url << endl;

  // Look up the Home object in the naming context
  // pointed to by the corbaname URL
  CORBA::Object_var home_obj
    = orb->string_to_object(home_url);
  
  assert(!CORBA::is_nil(home_obj));

  // Perform a safe downcast
  it::mokabyte::eai::my::ejb::CalculatorHome_var home
    = it::mokabyte::eai::my::ejb::CalculatorHome::_narrow(home_obj.in());

  assert(!CORBA::is_nil(home));

  // Create a Remote EJB reference
  it::mokabyte::eai::my::ejb::Calculator_var calc = home->create();

  CORBA::String_var home_ior = orb->object_to_string(calc);
  cout << "Calculator IOR is: " << home_ior << endl;

  

  CORBA::WStringValue* msg1 = new CORBA::WStringValue(L"Hello CORBA World");

  // Send our message
  wcout << L"Lunghezza del messaggio: " << msg1->_boxed_in() << L" = " << calc->getStringLength(msg1) << endl;
    
  // Tell the application server we won't use this
  // EJB reference any more
  calc->remove();

  cout << "Done" << endl;
}

/**
 * Simple main method that checks arguments, creates an
 * ORB, and handles exceptions.
 */
int
main(int argc, char* argv[])
{
  int exit_code = 0;
  CORBA::ORB_var orb;

  try {
	  //
  // Create an ORB
  orb = CORBA::ORB_init(argc, argv);

  // Check the arguments
  if (argc != 2) {
    cerr << "Usage: Client <corbaname URL of Calculator>" << endl;
    return 1;
  }


  run(orb, argv[1]);

} catch(const CORBA::Exception& ex) {
  // Handle any CORBA related exceptions
  cerr << ex << endl;
  exit_code = 1;
}

  return exit_code;
}
