import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

import it.mokabyte.eai.my.config.ConfiguratorBean;
/**
 * <p>Title: ConfiguratorConsole</p>
 * <p>Description: Configurator console</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author S: Rossini
 * @version 1.0
 */
public class ConfiguratorConsole extends JFrame {

  /* Componenti interfaccia grafica */
  /** Current dir */
  private String currentDir = null;
  /** Complete File name */
  private String fileName = null;
  /** File Name */
  private static final String FILE_NAME = "setenv_eai";
  /** File extension (.bet or .sh) */
  private String extension = null;
  /** File */
  private File file = null;
  /** Il bottone per il clear del messaggio da inviare */
  private JButton bClear = null;
  /** Il bottone per inviare il messagggio */
  private JButton bCommit = null;
  /** Il bottone per uscire senza salvare */
  private JButton bCancel = null;
  /** Il bottone di HELP ON LINE */
  private JButton bHelpOnLine = null;
  /** La TextField per inserire i nomi degli Host */
  private JTextField[] tfHost = null;
  /** La TextField per inserire i valori delle variabili d'ambiente */
  private JTextField[] tfEnv = null;
  /** Logo file name */
  private static final String LOGO_FILE_NAME = "images/moka_eai_logo.gif";
  /** Environment settings */
  private static final String[] strEnv = {
      "JAVA_HOME",
      "ANT_HOME",
      "JBOSS_DIST",
      "TOMCAT_HOME",
      "WLS_HOME",
      "AXIS_DIST",
      "AXIS_PATH",
      "MICO_HOME"
  };
  /** Label Host values */
  private static final String[] strHost = {
      "HOST_A (TOMCAT)[8080]:",
      "HOST_B (WLS)[7001]:",
      "HOST_C (IIS)[80]:",
      "HOST_D (MICO)[1666]:"
  };
  /** SET to write into file */
  String[] strHosts = {
      "HOST_A",
      "HOST_B",
      "HOST_C",
      "HOST_D"};
  /** Font*/
  private static Font font = new Font("Arial", Font.BOLD, 16);
  /** Titolo Frame & Message Dialog */
  private static final String TITLE = "Configurator Console 1.0";
  /** enable/disable INTERNAL Debug */
  private static final boolean DEBUG = true;
  /** enable/disable INTERNAL Deep Debug */
  private static final boolean DEEP_DEBUG = false;

  /**
   * Metodo main
   * @param args  i nomi JNDI del topic connection factory e del topic d'interesse (opzionali)
   */
  public static void main(String[] args) {
    new ConfiguratorConsole().go();
  }

  /**
   * Costruttore
   * @param topicConnectionFactoryName JNDI name del Connection Factory
   * @param topicName JNDI name del topic
   */
  public ConfiguratorConsole() {
    super(TITLE);
    this.currentDir = System.getProperty("user.dir");

    if (System.getProperty("os.name").startsWith("Windows")) {
      this.extension = ".bat";
    }
    else {
      this.extension = ".sh";
    }
    System.out.println("Configurator: currentDir["+this.currentDir+"]...");
    System.out.println("Configurator: extension["+this.extension+"]...");
  }

  /**
   * Avvio configurazione
   */
  public void go() {

    System.out.println("ConfiguratorConsole: go...");

    try {
     System.out.println("ConfiguratorConsole: buildGUI...");
      this.buildGUI();

      System.out.println("ConfiguratorConsole: loading environment...");
      this.loadFileEnv();
    }
    catch (Exception ex) {
      System.out.println("Configurator.go: # Exception # :" + ex.getMessage());
      ex.printStackTrace();
      javax.swing.JOptionPane.showMessageDialog(null, ex.getMessage(), TITLE,
                                                javax.swing.JOptionPane.
                                                ERROR_MESSAGE);
    }
  }

  /**
   * Load File ENVIRONMENT
   */
  private void loadFileEnv() {

    File file = null;
    FileInputStream fis = null;
    Properties p = null;

    file = new File("environment.properties");
    System.out.println("ConfiguratorBean.loadFileEnv: il file["+file.getAbsolutePath()+"] esiste...");

    if (!file.exists()) {
      String err=" # ERROR # File["+file.getAbsolutePath()+"] NON ESISTE !";
      javax.swing.JOptionPane.showMessageDialog(null, err, TITLE,
                                                javax.swing.JOptionPane.
                                                ERROR_MESSAGE);
      System.out.println("Configurator.loadFileEnv: " + err);
    }

    try {
      System.out.println("Configurator.loadFileEnv: il file["+file.getAbsolutePath()+"] esiste...");
      p = new Properties();
      fis = new FileInputStream(file);
      p.load(fis);
      System.out.println("Configurator.loadFileEnv: props["+p+"] read from file prop ...");
      if (p == null) {
        String err=" # ERROR # File["+file.getAbsolutePath()+"] ESISTE MA CORROTTO !";
        javax.swing.JOptionPane.showMessageDialog(null, err, TITLE,
                                                  javax.swing.JOptionPane.
                                                  ERROR_MESSAGE);
        System.out.println("Configurator.loadFileEnv: " + err);
      }

      for (int i = 0; i < strHosts.length; ++i) {
        System.out.println("Configurator: loading for ["+strHosts[i]+"] value["+ p.getProperty(strHosts[i])+"]...");
        if (p.getProperty(strHosts[i]) != null) {
          tfHost[i].setText(p.getProperty(strHosts[i]));
        }
      }
      for (int i = 0; i < strEnv.length; ++i) {
        System.out.println("Configurator: loading for ["+strEnv[i]+"] value["+ p.getProperty(strEnv[i])+"]...");
        if (p.getProperty(strEnv[i]) != null) {
          tfEnv[i].setText(p.getProperty(strEnv[i]));
        }
      }

      // MICO HOME  solo per linux
      if (System.getProperty("os.name").startsWith("Windows")) {
        tfEnv[strEnv.length-1].setText("");
        tfEnv[strEnv.length-1].setEditable(false);
      }

      fis.close();
    }
    catch (Exception e) {
      System.out.println("Configurator: # Exception # :" + e.getMessage());
    }
    finally {
      if (fis != null) {
        try {
          fis.close();
        }
        catch (Exception e) {}
      }
    }
  }

  /**
   * Costruzione dell'interfaccia grafica
   */
  private void buildGUI() {

    // inizializzazione componenti GUI
    tfEnv = new JTextField[strEnv.length];
    tfHost = new JTextField[strHost.length];

    for (int i = 0; i < strHost.length; ++i) {
      tfHost[i] = new JTextField();
      // tfHost[i].setFont(font);
      tfHost[i].setText("localhost");
    }

    // pulsanti
    bClear = new JButton("CLEAR");
    bCommit = new JButton("COMMIT");
    bCancel = new JButton("EXIT");
    bHelpOnLine = new JButton("HELP");

    // Logo
    System.out.println("Configurator: loading image[" + LOGO_FILE_NAME + "]...");
    ImageIcon img = new ImageIcon(LOGO_FILE_NAME);
    JLabel picture = new JLabel(img);

    // Layout: Num righe host + num righe env + 3(per i bottoni e le due intestazioni)
    int numRighe = strHost.length + strEnv.length + 3;
    int numCols = 2;
    System.out.println("Configurator: rows["+numRighe+"]-cols["+numCols+"]...");
    JPanel p = new JPanel();
    p.setLayout(new GridLayout(numRighe, numCols, 10, 10));

    Label lbl = null;
    // Header
    lbl = new Label("ENV");
    lbl.setFont(font);
    p.add(lbl);
    lbl = new Label("VAL");
    lbl.setFont(font);
    p.add(lbl);

    // ENV
    for (int i = 0; i < strEnv.length; ++i) {
      tfEnv[i] = new JTextField("...");
    }

    // MICO HOME  solo per Linux
    if (System.getProperty("os.name").startsWith("Windows")) {
      strEnv[strEnv.length-1]=strEnv[strEnv.length-1]+"(solo per Linux)";
    }

    for (int i = 0; i < strEnv.length; ++i) {
      p.add(new Label(strEnv[i]));
      p.add(tfEnv[i]);
    }


    // Header
    lbl = new Label("HOST Name/IP [TCP PORT]");
    lbl.setFont(font);
    p.add(lbl);
    lbl = new Label("VALORE HOST");
    lbl.setFont(font);
    p.add(lbl);

    // HOST
    for (int i = 0; i < strHost.length; ++i) {
      p.add(new Label(strHost[i]));
      p.add(tfHost[i]);
    }

    // p.add(bClear);
    p.add(bHelpOnLine);
    p.add(bCommit);

    // ADD to contentPane
    this.getContentPane().add(picture, BorderLayout.NORTH);
    this.getContentPane().add(p, BorderLayout.CENTER);
    this.getContentPane().add(bCancel, BorderLayout.SOUTH);

    // il bottone send invia il messaggio contenuto nel TextField
    bCommit.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        // avvia la congigurazione
        autoMagicallyConfiguration();
      }
    });

    // il bottone clear ripulisce la TextField
    bClear.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        for (int i = 0; i < strHosts.length; ++i) {
          tfHost[i].setText("");
        }
      }
    });

    // il bottone cancel esce dal programma
    bCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.exit( -1);
      }
    });

    // il bottone di HELP ON LINE
    bHelpOnLine.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {

        String msg = "O.S: " + System.getProperty("os.name") + " - WORK_DIR: " +
            currentDir + "\n" +
            "Inserire il valore delle variabili d'ambiente d'interesse\n"+
            "Inserire il nome o l'indirizzo IP delle macchine Server(localhost per la macchina locale)\n"+
            "Per il deploy eseguire il batch [" + currentDir + File.separator+"ant"+File.separator+"deployer"+extension+"]\n" +
            "I batch di test sono nella directory [" + currentDir +File.separator+"batch]\n"+
            "La documentazione e' disponibile in [" + currentDir + File.separator+"doc]\n" +
            "BUON LAVORO!\n";
        javax.swing.JOptionPane.showMessageDialog(null, msg, "Console Config",
                                                  javax.swing.JOptionPane.
                                                  INFORMATION_MESSAGE);
      }
    });

    // l'evento di chiusura della finestra genera la chiusura della connessione JMS
    this.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        System.out.println("windowClosing : ...");
        System.exit(0);
      }
    });

    // Dimensioni e posizionamento
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    this.setSize(screenSize.width * 2 / 3, screenSize.height * 2 / 3);
    Dimension frameSize = this.getSize();
    this.setLocation( (screenSize.width - frameSize.width) / 2,
                     (screenSize.height - frameSize.height) / 2);

    System.out.println("Configurator: content pane[" +
                       this.getContentPane().getLayout());
    this.setVisible(true);
  }

  /**
   * configurazione
   */
  private void autoMagicallyConfiguration() {

    File inputFile = null;
    File outputFile = null;
    BufferedReader br = null;

    try {

      System.out.println("\n\t>>> STEP 1: CHECK envinroment ...");

      // First of all: CHECK Enironment values ...
      for (int i = 0; i < strEnv.length; ++i) {
        System.out.println("Configurator.autoMagicallyConfiguration: CHECK FOR["+tfEnv[i].getText()+"]...");
        if(!tfEnv[i].getText().equals("") && !new File(tfEnv[i].getText()).exists()){
          String msg="Valore ["+tfEnv[i].getText()+"] ERRATO della variabile ["+strEnv[i]+"] !";
          javax.swing.JOptionPane.showMessageDialog(null,
              msg,
              "Console Config",
              javax.swing.JOptionPane.INFORMATION_MESSAGE);
          return;
        }
      }

      System.out.println("...5% ...");

      // =====< SETENV - BAT / SHELL >=====
      System.out.println("\n\t>>> STEP 2: creazione dei file [setenv_eai."+this.extension+"]...");

      outputFile = new File(currentDir + "/ant/setenv_eai"+this.extension);
      inputFile = new File(currentDir + "/ant/_template"+this.extension);

      this.createScriptFile(inputFile,outputFile);

      System.out.println("...10% ...");

      outputFile = new File(currentDir + "/setenv_eai"+this.extension);
      inputFile = new File(currentDir + "/ant/_template"+this.extension);

      this.createScriptFile(inputFile,outputFile);

      System.out.println("...15% ...");

      // =====< PROPERTIES >=====
      System.out.println("\n\t>>> STEP 3: creazione del file [setenv_eai.properties]...");
      this.createPropertiesFile(new File(this.currentDir+File.separator+"classes"+File.separator+FILE_NAME+".properties"),true);

      System.out.println("...20% ...");

      System.out.println("\n\t>>> STEP 4: creazione/update del file [environment.properties]...");
      this.createPropertiesFile(new File(this.currentDir+File.separator+"environment.properties"),false);

      System.out.println("...25% ...");

      int choice = JOptionPane.YES_OPTION;
      if(new File(currentDir + File.separator+"bin").exists()){
        choice = javax.swing.JOptionPane.showConfirmDialog(null,
            "Rebuild ?", "choose one", JOptionPane.YES_NO_OPTION);
      }

      if (choice == JOptionPane.YES_OPTION) {
        System.out.println("\n\t>>> STEP 5: esecuzione Builder ...");
        // =====< EXEC >=====
        this.callExec();
      }

      System.out.println("...100% ...");

      String msg="Configurazione avvenuta con successo!";
      javax.swing.JOptionPane.showMessageDialog(null,msg,"Console Config",
                                                javax.swing.JOptionPane.INFORMATION_MESSAGE);

    }catch (Exception ex) {
      javax.swing.JOptionPane.showMessageDialog(null, ex.getMessage(),
                                                "Console Config",
                                                javax.swing.JOptionPane.ERROR_MESSAGE);
      ex.printStackTrace();
    }
  }


  /**
     * createScriptFile
     * @param inputFile
     * @param outputFile
     */
   private void createScriptFile(File inputFile, File outputFile) throws java.io.IOException{

     PrintWriter pw = null;
     BufferedReader br = null;
     String text = null;

     System.out.println("Configurator.createScriptFile: <<< input file["+inputFile.getAbsolutePath()+"]...");
     System.out.println("Configurator.createScriptFile: >>> output file["+outputFile.getAbsolutePath()+"]...");

     if (outputFile.exists()) {
       outputFile.delete();
     }
     pw = new PrintWriter(new BufferedWriter(new FileWriter(outputFile)));

     if (System.getProperty("os.name").startsWith("Windows")) {
       pw.println("rem " + new java.util.Date().toString() +
                  ": AUTOGENERATED by ConfiguratorConsole MOKA EAI");
     }
     else {
       pw.println("# " + new java.util.Date().toString() +
                  ": AUTOGENERATED by ConfiguratorConsole MOKA EAI ...");
     }
     pw.flush();

     if (System.getProperty("os.name").startsWith("Windows")) {
       pw.println("SET WORK_DIR=" + currentDir);
       pw.println("SET MY_CLASSES=" + currentDir + File.separator +
                  "classes");
     }
     else {
       pw.println("export WORK_DIR=" + currentDir);
       pw.println("export MY_CLASSES=" + currentDir + File.separator +
                  "classes");
     }
     pw.flush();
     for (int i = 0; i < strHosts.length; ++i) {
       if (System.getProperty("os.name").startsWith("Windows")) {
         pw.println("SET " + strHosts[i] + "=" + tfHost[i].getText());
       }
       else {
         pw.println("export " + strHosts[i] + "=" + tfHost[i].getText());
       }
       pw.flush();
     }
     for (int i = 0; i < strEnv.length; ++i) {
       if (System.getProperty("os.name").startsWith("Windows")) {
         pw.println("SET " + strEnv[i] + "=" + tfEnv[i].getText());
       }
       else {
         pw.println("export " + strEnv[i] + "=" + tfEnv[i].getText());
       }
       pw.flush();
     }

     br = new BufferedReader(new FileReader(inputFile));
     while ( (text = br.readLine()) != null) {
       if (DEBUG)
         System.out.println(text);
       pw.println(text);
       pw.flush();
     }
     br.close();
     pw.close();
   }

  /**
   * createPropertiesFile
   * @param file
   * @throws java.io.IOException
   */
  private void createPropertiesFile(File file, boolean b) throws java.io.IOException{

    PrintWriter pw =null;
    String text=null;

    if (file.exists())file.delete();
    System.out.println("Configurator.createPropertiesFile: file["+file.getAbsolutePath()+"]...");
    pw = new PrintWriter(new BufferedWriter(new FileWriter(file.getAbsolutePath(),file.exists())));
    if(DEBUG) System.out.println("Configurator: currentDir["+currentDir+"]...");
    if(b){
      if (System.getProperty("os.name").startsWith("Windows")) {
        String winCurrentDir = java.util.regex.Pattern.compile("\\\\").matcher(this.
            currentDir).replaceAll("\\\\\\\\");
        pw.println("WORK_DIR=" + winCurrentDir);
        pw.flush();
      }
      else {
        pw.println("WORK_DIR=" + this.currentDir);
        pw.flush();
      }
    }
    for (int i = 0; i < strHosts.length; ++i) {
        pw.println(strHosts[i] + "=" + tfHost[i].getText());
        pw.flush();
    }

    for(int i=0; i<strEnv.length; ++i){
      if (System.getProperty("os.name").startsWith("Windows")) {
        String winEnv = java.util.regex.Pattern.compile("\\\\").matcher(tfEnv[i].getText()).replaceAll("\\\\\\\\");
        pw.println(strEnv[i] + "=" + winEnv);
        pw.flush();
      }
      else {
        pw.println(strEnv[i] + "=" + tfEnv[i].getText());
        pw.flush();
      }
    }
    pw.close();
  }

  /**
   * callExec
   * @throws java.io.IOException
   * @throws InterruptedException
   */
  private void callExec() throws java.io.IOException,InterruptedException {

    BufferedReader br = null;

    // =====< EXEC Builder BAT / Shell >=====
    String cmd = currentDir + "/ant/builder" + this.extension;
    System.out.println("Configurator: executing [" + cmd + "]...");

    File f = new File(cmd);
    if (!f.exists()) {
      System.out.println("Configurator: # ERROR # File[" +
                             f.getAbsolutePath() + "] NON ESISTE!");
    }
    else {
      if (DEBUG) System.out.println("Configurator: FILE[" + f.getAbsolutePath() +
                               "] esiste...");
    }

    if(DEBUG) System.out.println("Configurator: FILE[" + f.getAbsolutePath() +
                               "] in esecuzione ...");
    // start command running
    Process proc = Runtime.getRuntime().exec(cmd);
    // get command's output stream and put a buffered reader input stream on it
    InputStream istr = proc.getInputStream();
    br = new BufferedReader(new InputStreamReader(istr));
    // set up list to capture command output lines
    ArrayList list = new ArrayList();
    // read output lines from command
    String str;
    while ( (str = br.readLine()) != null) {
      if (DEBUG)System.out.println(str);
      list.add(str);
    }

    System.out.println("...40% ...");

    // wait for command to terminate
    if (DEBUG) System.out.println("Executor.runCommand : BEFORE waiting process's end ...");
    proc.waitFor();
    if (DEBUG)System.out.println("Executor.runCommand : process OVER!");

    System.out.println("...90% ...");

    // check its exit value
    String msg = "CONFIGURAZIONE terminata in modo anomalo";
    if (proc.exitValue() != 0) {
      if (DEBUG)System.err.println("exit value was non-zero");
          javax.swing.JOptionPane.showMessageDialog(null, msg,
          "Console Config",
          javax.swing.JOptionPane.INFORMATION_MESSAGE);
      return;
    }

    System.out.println("Configurator: esecuzione["+f.getAbsolutePath()+"] avvenuta...");

    // dump file executed
    String[] res = ( (String[]) list.toArray(new String[0]));
    for (int i = 0; i < res.length; ++i) {
      if (DEEP_DEBUG)System.out.println("" + res[i]);
    }
  }

 }