package it.mokabyte.eai.my.corba;

import java.util.Properties;

import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import org.omg.PortableServer.POA;

import eai.Calcolatrice;
import eai.CalcolatriceHelper;
/**
 * <p>Title: CalculatorServer</p>
 * <p>Description: CORBA Server</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author not attributable
 * @version 1.0
 */
public class CalcolatriceServer{

  public static void main(String args[]){

    try{
      // create and initialize the ORB
      ORB orb = ORB.init(args, null);

      System.out.println("CalcolatriceServer: ORB["+orb+"]-class["+orb.getClass().getName()+"]...");
      // Get reference to rootpoa & activate the POAManager
      POA rootpoa = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
      rootpoa.the_POAManager().activate();
      System.out.println("CalcolatriceServer: POA ["+rootpoa +"]-class["+rootpoa .getClass().getName()+"]...");

      // create servant and register it with the ORB
      CalcolatriceImpl servant = new CalcolatriceImpl();
      servant.setORB(orb);
      System.out.println("CalcolatriceServer: Servant ["+servant+"]-class["+servant.getClass().getName()+"]...");

      // get object reference from the servant
      org.omg.CORBA.Object ref = rootpoa.servant_to_reference(servant);
      Calcolatrice href = CalcolatriceHelper.narrow(ref);
      System.out.println("CalcolatriceServer: Calcolatrice["+href+"]-class["+href.getClass().getName()+"]...");

      // get the root naming context (Chiave NameService ==> NameService: servizio di naming persistent - TNameService: servizio di naming transient)
      System.out.println("CalcolatriceServer: resolving initial references NameService ...");
      org.omg.CORBA.Object objRef = orb.resolve_initial_references("NameService");
      System.out.println("CalcolatriceServer: RootNamingContext["+objRef +"]-class["+objRef .getClass().getName()+"]...");

      // Use NamingContextExt which is part of the Interoperable
      // Naming Service specification.
      NamingContextExt ncRootRef = NamingContextExtHelper.narrow(objRef);
      System.out.println("CalcolatriceServer: NamingContextExt ["+ncRootRef+"]-class["+ncRootRef.getClass().getName()+"]...");

      NameComponent path[] = null;
      NamingContext ncChild = null;

      try{
        // Get the root naming context
        path = ncRootRef.to_name("eai");
        System.out.println("CalcolatriceServer: ready to bind with NameComponent["+path+"]...");
        ncChild = ncRootRef.bind_new_context(path);
      }catch(AlreadyBound ab){
        System.out.println("CalcolatriceServer: 'eai' already bound ...");
      }

      path = ncRootRef.to_name("eai/calcolatrice");
      // path = ncChild.to_name("calcolatrice");

      // bind the Object Reference in Naming
      System.out.println("CalcolatriceServer: ready to bind with NameComponent["+path+"]...");
      ncRootRef.rebind(path, href);
      // ncChild.rebind(path, href);

      System.out.println("CalcolatriceServer: bind done ! NameComponent ["+path+"]...");

      // wait for invocations from clients
      orb.run();
      System.out.println("CalcolatriceServer: ready and waiting ...");
    }
    catch (Exception e){
      System.err.println("ERROR: " + e);
      e.printStackTrace(System.out);
    }
    System.out.println("HelloServer Exiting ...");
  }
}
