package it.mokabyte.eai.my.delegate.jms;

// JMS
import javax.jms.*;

import it.mokabyte.eai.my.delegate.CalculatorBusinessDelegate;
import it.mokabyte.eai.my.locator.MyServiceLocator;
import it.mokabyte.eai.my.exception.ServiceLocationException;
import it.mokabyte.eai.my.exception.BusinessDelegateException;
import it.mokabyte.eai.my.valueobject.MailOM;
import it.mokabyte.eai.my.util.Defines;
import it.mokabyte.eai.my.util.SimpleTrace;
 /**
  * <p>Title: MyBusinessDelegate </p>
  * <p>Description: Business Delegate Applicazione A</p>
  * <p>Copyright: Copyright (c) 2003</p>
  * <p>Company: </p>
  * @author S. Rossini
  * @version 1.0
  */
public class CalculatorBdJMSImpl implements CalculatorBusinessDelegate {

  /** Il reference al Queue Connection Factory */
  private QueueConnectionFactory queueConnectionFactory = null;

  /** La destinazione queue JMS */
  private Queue queue = null;

  /** La mail object value */
  private MailOM param = null;

  /**
   * Constructor
   * @throws ServiceLocationException
   */
  public CalculatorBdJMSImpl(MailOM mail) {
    SimpleTrace.trace("CalculatorBdJMSImpl::CalculatorBdJMSImpl: mail["+mail+"]...");
    this.param=mail;
  }

  /**
    * This method get the string length of the parameter
    * @return
    * @throws BusinessDelegateException
    */
  public int getLength(String param) throws BusinessDelegateException {

      // Delegate to the Calculator MDB
      try {
          if(this.queueConnectionFactory==null){
            this.queueConnectionFactory=(QueueConnectionFactory)MyServiceLocator.getJmsConnectionFactory(Defines.SERVICE_JMS_FACTORY);
            SimpleTrace.trace("CalculatorBdJMSImpl.getLengthByJMS: calculator Queue Connection Factory["+this.queueConnectionFactory+"] ...");
            SimpleTrace.trace("CalculatorBdJMSImpl.getLengthByJMS: param["+param+"] ...");
          }
          QueueConnection queueConnection = this.queueConnectionFactory.createQueueConnection();
          QueueSession queueSession = queueConnection.createQueueSession(false,Session.AUTO_ACKNOWLEDGE);
          if(this.queue==null){
            this.queue=(Queue)MyServiceLocator.getJmsDestination(Defines.SERVICE_JMS_QUEUE);
            SimpleTrace.trace("CalculatorBdJMSImpl.getLengthByJMS: calculator Queue["+this.queue+"] ...");
          }
          SimpleTrace.trace("CalculatorBdJMSImpl.getLengthByJMS: QCF["+this.queueConnectionFactory+"] QUEUE["+this.queue+"]...");
          QueueSender queueSender = queueSession.createSender(queue);
          SimpleTrace.trace("CalculatorBdJMSImpl.getLengthByJMS: QueueSender["+queueSender+"]...");
          ObjectMessage messageToQueue = queueSession.createObjectMessage();
          messageToQueue.setObject(param);
          // messageToQueue.setJMSReplyTo(this.queue);
          queueSender.send(messageToQueue);
          SimpleTrace.trace("CalculatorBdJMSImpl.getLengthByJMS: messaggio inviato...");
          queueConnection.close();
          return 0; // dummy retcode
      } catch(ServiceLocationException sle){
          sle.printStackTrace();
          throw new BusinessDelegateException(sle);
      } catch (JMSException jmse) {
          jmse.printStackTrace();
          throw new BusinessDelegateException(jmse);
      }
  }
}