package it.mokabyte.eai.my.ejb;

import javax.ejb.MessageDrivenBean;
import javax.ejb.MessageDrivenContext;
import javax.ejb.EJBException;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.ObjectMessage;
import javax.jms.TextMessage;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.mail.*;
import javax.mail.internet.MimeMessage;

import javax.mail.*;
import javax.activation.*;

import it.mokabyte.eai.my.locator.MyServiceLocator;
import it.mokabyte.eai.my.exception.ServiceLocationException;
import it.mokabyte.eai.my.util.Defines;
import it.mokabyte.eai.my.valueobject.MailOM;
/**
 * <p>Title: CalculatorMDB</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author : S.Rossini
 * @version 1.0
 */
public class CalculatorMDB implements MessageDrivenBean, MessageListener {

   private MessageDrivenContext ctx = null;
   private QueueConnection conn;
   private QueueSession session;

   /**
     * Constructor
     */
   public CalculatorMDB(){
      System.out.println("CalculatorMDB::CalculatorMDB : ...");
   }

   /**
    * Set the MessageDrivenContext
    * Lookup the QueueConnectionFactory and create and start the session
    * @param ctx MessageDrivenContext
    */
   public void setMessageDrivenContext(MessageDrivenContext ctx){
      this.ctx = ctx;
      System.out.println("CalculatorMDB.setMessageDrivenContext : ctx["+this.ctx+"]...");
   }

   /**
    * ejbCreate method, declared as public (but not final or
    * static), with a return type of void, and with no arguments.
    */
   public void ejbCreate() {
      System.out.println("CalculatorMDB.ejbCreate : ...");
      try{
        InitialContext iniCtx = new InitialContext();
        Object tmp = iniCtx.lookup("java:comp/env/jms/QCF");
        QueueConnectionFactory qcf = (QueueConnectionFactory) tmp;
        conn = qcf.createQueueConnection();
        session = conn.createQueueSession(false,QueueSession.AUTO_ACKNOWLEDGE);
        conn.start();
        System.out.println("CalculatorMDB.ejbCreate :connection started!");
      }
      catch(NamingException ne){
         ne.printStackTrace();
         throw new EJBException("CalculatorMDB.ejbCreate : creation failed due NamingException : " + ne.getMessage());
      }
      catch(Exception ex){
         ex.printStackTrace();
         throw new EJBException("CalculatorMDB.ejbCreate : creation failed : " + ex.getMessage());
      }
   }

   /**
     * ejbRemove method, declared as public (but not final or
     * static), with a return type of void, and with no arguments.
     */
   public void ejbRemove(){
      System.out.println("CalculatorMDB.ejbRemove : ...");
      ctx = null;
      try{
         if( session != null ){
            session.close();
         }
         if( conn != null ){
            conn.close();
         }
      }
      catch(JMSException ex){
         ex.printStackTrace();
         throw new EJBException("CalculatorMDB.ejbRemove : closing resources failed : " + ex.getMessage());
      }
   }

   /**
     * onMessage method, declared as public (but not final or static),
     * with a return type of void, and with one argument of type javax.jms.Message.
     * Casts the incoming Message to a TextMessage and displays
     * the text.
     *
     * @param inMessage	the incoming message
     */
   public void onMessage(Message msg){

      int result;

      try{

         String CTX_FACTORY = "org.jnp.interfaces.NamingContextFactory";
         String PROVIDER_URL = "jnp://localhost:1099";
         String JNDI_NAME = "MokaCalculator";

         System.out.println("CalculatorMDB.onMessage: msg["+ msg.getClass().getName()+"]...");
         ObjectMessage om = (ObjectMessage)msg;
         MailOM mail = (MailOM)om.getObject();
         String parola = mail.getBodyMail();

         // Use Calcualator EJB
         java.util.Properties properties = new java.util.Properties();
         properties.put(javax.naming.Context.INITIAL_CONTEXT_FACTORY, CTX_FACTORY);
         properties.put(javax.naming.Context.PROVIDER_URL, PROVIDER_URL);
         javax.naming.Context ctx = new javax.naming.InitialContext(properties);

         //look up jndi name
         System.out.println("CalculatorMDB.onMessage: Lookup of ["+JNDI_NAME+"]...");

         Object ref = ctx.lookup(JNDI_NAME);
         System.out.println("CalculatorService.getLength: Lookup done : obj["+ref+"]...");
         System.out.println("CalculatorService.getLength: Lookup done : obj class["+ref.getClass().getName()+"]...");

         //cast to Home interface
         CalculatorHome home = (CalculatorHome)javax.rmi.PortableRemoteObject.narrow(ref, CalculatorHome.class);
         System.out.println("CalculatorService.getLength: Narrow done : home["+home+"]...");
         System.out.println("CalculatorService.getLength: Narrow done : home["+home.getClass().getName()+"]...");

         Calculator calc = home.create();
         result = calc.getStringLength(parola);
         mail.setBodyMail("La lunghezza di ["+parola+"] e' di " + result+".");

         /* --- Reply to JMS
         Queue dest = (Queue) msg.getJMSReplyTo();
         System.out.println("CalculatorMDB.onMessage: dest["+dest+"]...");
         this.sendJMSReply(result, dest);
         --- */

         // Reply by email
         this.sendEmailReply(mail);
      }
      catch(Exception ex){
         System.out.println(" # EXCEPTION # CalculatorMDB.onMessage !");
         ex.printStackTrace();
      }
   }

   /**
    * Echo back the message received
    * @param text
    * @param dest
    * @throws JMSException
    */
   private void sendJMSReply(int result, Queue dest)throws JMSException {
        System.out.println("CalculatorMDB.sendReply to JMS destination["+dest+"]...");
        QueueSender sender = session.createSender(dest);
        TextMessage tm = session.createTextMessage("" + result);
        sender.send(tm);
        sender.close();
        System.out.println("CalculatorMDB.sendReply to JMS destination["+dest+"] done!");
   }

   private void sendEmailReply(MailOM result) throws NamingException,javax.mail.MessagingException{

        // String recipient="srossini@mokabyte.it";

        String risorsa="java:comp/env/mail/MokaMailSession";

        String recipient=result.getRecipient();

        System.out.println("CalculatorMDB.sendNotice: sending to ["+recipient+"]...");
        javax.naming.Context initial = new javax.naming.InitialContext();
        System.out.println("CalculatorMDB.sendNotice: BEFORE LOOKUP ["+risorsa+"]...");
        javax.mail.Session session =(javax.mail.Session) initial.lookup(risorsa);
        System.out.println("CalculatorMDB.sendNotice: Mail session found [" + session + "]...");
        javax.mail.Message msg = new javax.mail.internet.MimeMessage(session);

        msg.setRecipients(javax.mail.Message.RecipientType.TO,
                          javax.mail.internet.InternetAddress.parse(recipient, false));

        System.out.println("CalculatorMDB.sendNotice: Mail from["+msg.getFrom()+"]...");
        System.out.println("CalculatorMDB.sendNotice: Mail replyTo["+msg.getReplyTo()+"]...");

        // MESSAGE SUBJECT
        msg.setSubject("Message from Calculator Service");

        java.text.DateFormat dateFormatter = java.text.DateFormat.getDateTimeInstance(java.text.DateFormat.LONG, java.text.DateFormat.SHORT);
        java.util.Date timeStamp = new java.util.Date();
        // MESSAGE BODY
        msg.setText("" + dateFormatter.format(timeStamp) + result.getBodyMail());
        System.out.println("CalculatorMDB.sendNotice: MSG[" + msg +"]... ");
        msg.setHeader("X-Mailer", "CalculatorMDB");
        msg.setSentDate(timeStamp);

        System.out.println("CalculatorMDB.sendNotice: BEFORE to send ["+msg+"]...");
        javax.mail.Transport.send(msg);
        System.out.println("CalculatorMDB.sendNotice: Mail SENT!");
    }
}














