package it.mokabyte.eai.my.locator;

import java.util.Properties;
import java.io.*;

import javax.naming.*;
import javax.rmi.PortableRemoteObject;
import javax.ejb.EJBHome;

import org.omg.CORBA.ORB;

import javax.jms.ConnectionFactory;
import javax.jms.Destination;

import it.mokabyte.eai.my.exception.ServiceLocationException;
import it.mokabyte.eai.my.util.SimpleTrace;
import it.mokabyte.eai.my.util.PublicLibraries;
import it.mokabyte.eai.my.config.ConfiguratorBean;
/**
 * <p>Title: MyServiceLocator</p>
 * <p>Description: classe di Service Locator per EJB & risorse JMS: permette di </p>
 * <li> localizzazione type unchecked location EJB
 * <li> localizzazione type unchecked locastion risorse JMS
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author S.Rossini
 * @version 1.0
 */
public class MyServiceLocator {

  /** Configration file */
  private static final String CONFIG_FILE_NAME = "/config/locator.xml";

  /** Service configurations array */
  private static ServiceConfig businessServices[] = null;

  /** singleton's private instance */
  private static MyServiceLocator me;

  static { me = new MyServiceLocator();}

  private MyServiceLocator(){
    try {
      SimpleTrace.trace("MyServiceLocator: costruzione ...");
      String configFileName=new ConfiguratorBean().getRootDir()+CONFIG_FILE_NAME;
      SimpleTrace.trace("MyServiceLocator: configFileName["+configFileName+"]...");
      SimpleTrace.trace("MyServiceLocator: reading configuration file[" + new File(configFileName).getAbsolutePath()+"]...");
      java.beans.XMLDecoder d = new java.beans.XMLDecoder( new BufferedInputStream( new FileInputStream(configFileName)));
      Object result = d.readObject();
      SimpleTrace.trace("MyServiceLocator: reading class["+result.getClass().getName()+"]...");

      Object o[] = (Object[])result;
      SimpleTrace.trace("MyServiceLocator: config slots found[" + o.length + "]...");

      businessServices = new ServiceConfig[o.length];

      for(int i=0; i<o.length; ++i){
       if(o[i] instanceof it.mokabyte.eai.my.locator.ServiceConfig){
        SimpleTrace.trace("MyServiceLocator: lettura slot ["+i+"]...");
        businessServices[i] = (ServiceConfig)o[i];
        SimpleTrace.trace("MyServiceLocator: slot ["+i+"]:"+businessServices[i]);
       }
       else{
        SimpleTrace.trace("ERROR # MyServiceLocator # WRONG TYPE: aspettavo["+ServiceConfig.class.getName()+"] trovato["+o[i].getClass().getName()+"]...");
       }
      }
      d.close();
    }catch(Exception e){
        SimpleTrace.trace("EXCEPTION # MyServiceLocator: " + e.getMessage());
        e.printStackTrace();
    }
  }

  /** Localizzazione type checked di una EJB Home dato un SERVICE ID
    * @param  service
    * @return EJBHome
    * @throws ServiceException
    */
  public static EJBHome getEjbHome(int serviceID) throws ServiceLocationException {

    try {
      SimpleTrace.trace("MyServiceLocator.getEjbHome: service ID["+serviceID+"]...");
      MyServiceLocator.check(serviceID);

      // Look up using the service name from defined constant
      ServiceConfig config = MyServiceLocator.businessServices[serviceID];
      SimpleTrace.trace("MyServiceLocator.getEjbHome: ServiceConfig["+config+"]...");

      Context ctx = getInitialContext(config.getInitialContextFactory(),config.getProviderUrl());
      SimpleTrace.trace("MyServiceLocator.getEjbHome: looking up for [" + config.getJndiName() + "]...");

      Object objref = ctx.lookup(config.getJndiName());
      SimpleTrace.trace("MyServiceLocator.getEjbHome: objref class["+objref.getClass().getName()+"] ...");
      SimpleTrace.trace("MyServiceLocator.getEjbHome: objref ["+objref+"] ...");
      SimpleTrace.trace("MyServiceLocator.getEjbHome: class to narrow : " + config.getHomeClass());

      // Narrow using the EJBHome Class from defined constant
      Object obj = PortableRemoteObject.narrow(objref,config.getHomeClass());
      SimpleTrace.trace("MyServiceLocator.getEjbHome: obj["+obj+"]-class["+obj.getClass().getName()+"]...");

      EJBHome home = (EJBHome)obj;
      SimpleTrace.trace("MyServiceLocator.getEjbHome: return home ["+home+"]-class["+home.getClass().getName()+"] ...");
      return home;
    }
    catch( NamingException ne ) {
      ne.printStackTrace();
      throw new ServiceLocationException(ne);
    }
    catch( ClassCastException cce ) {
      cce.printStackTrace();
      throw new ServiceLocationException(cce);
    }
    catch( Exception ex ) {
      ex.printStackTrace();
      throw new ServiceLocationException(ex);
    }
  }

  /** Localizzazione type checked di una EJB Home dato un SERVICE ID
    * @param  service
    * @return EJBHome
    * @throws ServiceException
    */
  public static Object getCorbaRef(int serviceID) throws ServiceLocationException {

    try {
      SimpleTrace.trace("MyServiceLocator.getCorbaRef: service ID["+serviceID+"]...");
      MyServiceLocator.check(serviceID);

      // Look up using the service name from defined constant
      ServiceConfig config = MyServiceLocator.businessServices[serviceID];
      SimpleTrace.trace("MyServiceLocator.getCorbaRef: ServiceConfig["+config+"]...");

      String[] args={};
      ORB orb = ORB.init(args, null);
      SimpleTrace.trace("MyServiceLocator.getObject: orb["+orb+"]-class[" + orb.getClass().getName()+"]...");

      Context ctx = getInitialContext(config.getInitialContextFactory(),config.getProviderUrl(),orb);
      SimpleTrace.trace("MyServiceLocator.getCorbaRef : looking up for [" + config.getJndiName() + "]...");

      Object objref = ctx.lookup(config.getJndiName());
      SimpleTrace.trace("MyServiceLocator.getCorbaRef: objref class["+objref.getClass().getName()+"] ...");
      SimpleTrace.trace("MyServiceLocator.getCorbaRef: objref ["+objref+"] ...");
      SimpleTrace.trace("MyServiceLocator.getCorbaRef: class to narrow : " + config.getHomeClass());

      // Narrow using the Class from defined constant
      Object obj = PortableRemoteObject.narrow(objref, config.getHomeClass());
      SimpleTrace.trace("MyServiceLocator.getCorbaRef: return obj["+obj+"]-class["+obj.getClass().getName()+"]...");

      return obj;
    }
    catch( NamingException ne ) {
      ne.printStackTrace();
      throw new ServiceLocationException(ne);
    }
    catch( ClassCastException cce ) {
      cce.printStackTrace();
      throw new ServiceLocationException(cce);
    }
    catch( Exception ex ) {
      ex.printStackTrace();
      throw new ServiceLocationException(ex);
    }
  }

   /**
   * Localizzazione type checked per Connection Factory JMS
   * @param CxtFact the ContextFactory class name
   * @param provUrl il provider URL
   * @param jndiName il nome JNDI
   * @return il Topic JMS Factory
   * @throws ServiceLocationException
   */
  public static ConnectionFactory getJmsConnectionFactory(int serviceID) throws ServiceLocationException {

     try {
      SimpleTrace.trace("MyServiceLocator.getJmsConnectionFactory: service ID["+serviceID+"]...");
      MyServiceLocator.check(serviceID);

      ServiceConfig config = (ServiceConfig)MyServiceLocator.businessServices[serviceID];
      SimpleTrace.trace("MyServiceLocator.getJmsConnectionFactory: ServiceConfig["+config+"]...");

      Context ctx = getInitialContext(config.getInitialContextFactory(),config.getProviderUrl());
      SimpleTrace.trace("MyServiceLocator.getJmsConnectionFactory: looking up for [" + config.getJndiName() + "]...");

      Object obj = ctx.lookup(config.getJndiName());
      SimpleTrace.trace("MyServiceLocator.getJmsConnectionFactory: objref ["+obj+"] - class["+obj.getClass().getName()+"] ...");

      ConnectionFactory connectionFactory = (ConnectionFactory)obj;
      SimpleTrace.trace("MyServiceLocator.getJmsConnectionFactory: connectionFactory ["+connectionFactory+"] - class["+connectionFactory.getClass().getName()+"] ...");

      SimpleTrace.trace("MyServiceLocator.getJmsDestination: returning ["+connectionFactory+"]...");
      return connectionFactory;
    }
    catch( NamingException ne ) {
      ne.printStackTrace();
      throw new ServiceLocationException(ne);
    }
    catch( Exception ex ) {
      ex.printStackTrace();
      throw new ServiceLocationException(ex);
    }
  }

  /**
    * Localizzazione type-checked per destinazioni JMS
    * Ritorna la destinazione relativa all'identificativo del servizio passato
    * @param  service l'identificativo del business object ricercato
    * @return l'oggetto Destination
    * @throws ServiceException
    */
  public static Destination getJmsDestination(int serviceID) throws ServiceLocationException {

     try {
      SimpleTrace.trace("MyServiceLocator.getJmsDestination: service ID["+serviceID+"]...");
      MyServiceLocator.check(serviceID);

      ServiceConfig config = (ServiceConfig)MyServiceLocator.businessServices[serviceID];
      SimpleTrace.trace("MyServiceLocator.getJmsDestination: ServiceConfig["+config+"]...");

      Context ctx = getInitialContext(config.getInitialContextFactory(),config.getProviderUrl());
      SimpleTrace.trace("MyServiceLocator.getJmsDestination: looking up for [" + config.getJndiName() + "]...");

      Object obj = ctx.lookup(config.getJndiName());
      SimpleTrace.trace("MyServiceLocator.getJmsDestination: objref ["+obj+"] - class["+obj.getClass().getName()+"] ...");

      Destination dest = (Destination)obj;
      SimpleTrace.trace("MyServiceLocator.getJmsDestination: return dest ["+dest+"] - class["+dest.getClass().getName()+"] ...");
      return dest;
    }
    catch( NamingException ne ) {
      ne.printStackTrace();
      throw new ServiceLocationException(ne);
    }
    catch( Exception ex ) {
      ex.printStackTrace();
      throw new ServiceLocationException(ex);
    }
  }

  /**
   * get the initial context with INITIAL_CONTEXT_FACTORY and PROVIDER_URL specified
   * @param ctxFacxtory
   * @param provUrl
   * @return default context
   * @throws NamingException
   */
  private static Context getInitialContext(String ctxFacxtory, String provUrl) throws NamingException {

      provUrl=mapLogicHostToConfiguredHost(provUrl);
      Properties properties = null;
      properties = new Properties();
      properties.put(Context.INITIAL_CONTEXT_FACTORY, ctxFacxtory);
      properties.put(Context.PROVIDER_URL, provUrl);
      Context ctx = new InitialContext(properties);
      PublicLibraries.printContextProperties(ctx);
      return ctx;
  }

  /**
   * get the initial context with INITIAL_CONTEXT_FACTORY and PROVIDER_URL specified
   * @param ctxFacxtory
   * @param provUrl
   * @return default context
   * @throws NamingException
   */
  private static Context getInitialContext(String ctxFacxtory, String provUrl,ORB orb) throws NamingException {

      provUrl=mapLogicHostToConfiguredHost(provUrl);
      Properties properties = null;
      properties = new Properties();
      properties.put("java.naming.corba.orb", orb);
      properties.put(Context.INITIAL_CONTEXT_FACTORY, ctxFacxtory);
      properties.put(Context.PROVIDER_URL, provUrl);
      Context ctx = new InitialContext(properties);
      PublicLibraries.printContextProperties(ctx);
      return ctx;
  }

  private static String mapLogicHostToConfiguredHost(String logicalHost){

    SimpleTrace.trace("MyServiceLocator.mapLogicHostToConfiguredHost: arrivato["+logicalHost+"]...");
    int begin = logicalHost.indexOf("//");
    SimpleTrace.trace("MyServiceLocator.mapLogicHostToConfiguredHost: BEGIN ["+begin+"]...");
    String header=logicalHost.substring(0,begin+2);
    SimpleTrace.trace("MyServiceLocator.mapLogicHostToConfiguredHost: HEADER ["+header+"]...");
    int end = logicalHost.indexOf(":",begin);
    SimpleTrace.trace("MyServiceLocator.mapLogicHostToConfiguredHost: END ["+end+"]...");
    String tailer=logicalHost.substring(end, logicalHost.length());
    SimpleTrace.trace("MyServiceLocator.mapLogicHostToConfiguredHost: TAILER ["+tailer+"]...");
    String res= logicalHost.substring(begin+2, end);
    SimpleTrace.trace("MyServiceLocator.mapLogicHostToConfiguredHost: LOGICAL HOST["+res+"]...");
    if(res.equalsIgnoreCase(ConfiguratorBean.HOST_A))
      res=new ConfiguratorBean().getHosta();
    else if(res.equalsIgnoreCase(ConfiguratorBean.HOST_B))
      res=new ConfiguratorBean().getHostb();
    else if(res.equalsIgnoreCase(ConfiguratorBean.HOST_C))
      res=new ConfiguratorBean().getHostc();
    else if(res.equalsIgnoreCase(ConfiguratorBean.HOST_D))
      res=new ConfiguratorBean().getHostd();
    else {
      SimpleTrace.trace(" # WARNING # MyServiceLocator.mapLogicHostToConfiguredHost: LOGICAL HOST["+res+"] # NOT MAPPED #...");
      res="localhost";
    }

    String ret=header+res+tailer;
    SimpleTrace.trace("MyServiceLocator.mapLogicHostToConfiguredHost: returning["+ret+"]...");
    return ret;
  }

  /**
   * Check that businessServices is well configured and service ID is valid
   * @param srvID
   * @throws ServiceLocationException
   */
  private static void check(int srvID) throws ServiceLocationException{
     if(businessServices == null) {
          throw new ServiceLocationException(new Exception("MyServiceLocator.check: CONFIGURATION BY ["+CONFIG_FILE_NAME+"] NOT DONE !"));
     }
     if(businessServices.length == 0) {
          throw new ServiceLocationException(new Exception("MyServiceLocator.check: CONFIGURATION ["+CONFIG_FILE_NAME+"] CONTAINS NO DATA !"));
     }
     if(srvID < 0 || srvID > businessServices.length){
         throw new ServiceLocationException(new Exception("MyServiceLocator.check: service ID ["+srvID+"] OUT OF BOUNDS !"));
     }
  }

  // debug method
  public static void printConfiguration() throws ServiceLocationException{

     if(businessServices == null) {
          throw new ServiceLocationException(new Exception("MyServiceLocator.getHome : CONFIGURATION ["+CONFIG_FILE_NAME+"] NOT DONE !"));
     }
     for(int i=0; i<businessServices.length; ++i){
      System.out.println(">>> "+i+" >>>" + businessServices[i]+"\n");
     }
  }

  // simple test
  public static void main(String args[]) {
    try{
      MyServiceLocator.printConfiguration();
    }catch(ServiceLocationException sle){
      SimpleTrace.trace("### MAIN ### " + sle.getMessage());
    }
  }

  /*
   public SOAPConnectionFactory getSoapConnectionFactory() throws ServiceLocationException {

       //  javax.xml.soap.SOAPConnectionFactory scf = javax.xml.soap.SOAPConnectionFactory.newInstance();
       return javax.xml.soap.SOAPConnectionFactory.newInstance();

       // Il client esegue :
       javax.xml.soap.SOAPConnectionFactory scf = MyserviceLocator.getInstance().getSoapConnectionFactory();
       javax.xml.soap.SOAPConnection connection = scf.createConnection();
 }

 public static Call getCallRpc(int serviceID) throws ServiceLocationException {

   try {
     SimpleTrace.trace("MyServiceLocator.getCallRpc: service ID["+serviceID+"]...");
     MyServiceLocator.check(serviceID);

     ServiceConfig config = (ServiceConfig)MyServiceLocator.businessServices[serviceID];
     SimpleTrace.trace("MyServiceLocator.getCallRpc: ServiceConfig["+config+"]...");

     // Create the Service object
     Service service = new Service();
     SimpleTrace.trace("MyServiceLocator.getCallRpc: Service["+ service +"]-class[" + service.getClass().getName() + "]...");

     // Create the Call object
     Call call = (Call) service.createCall();
     SimpleTrace.trace("MyServiceLocator.getCallRpc: call["+call+"]-class["+call.getClass().getName()+"]...");

     // Set the TargetEndpointAddress
     call.setTargetEndpointAddress(new java.net.URL(config.getEndpointURL()));
     SimpleTrace.trace("MyServiceLocator.getCallRpc: TargetEndpointAddress["+call.getTargetEndpointAddress()+"]...");

     call.setOperationName(new QName(config.getEndpointURL(), config.getMethod()));
     SimpleTrace.trace("MyCalculatorClient_DII: OperationName["+call.getOperationName()+"]...");

     return call;
   }
   catch( Exception ex ) {
       ex.printStackTrace();
       throw new ServiceLocationException(ex);
   }
 }
*/

}
